'Update Let's Encrypt Certificates in ioBroker ber simpleAPI oder Command Line
'07.05.2017 by Daniel Matschull
'Version 1.00

'################################################################
'#                           Settings                           #
'################################################################

'Hostadresse und Port der ioBroker simpleAPI
const host = "https://localhost:8087/"

'Update-Methode
const UpdMethod = "SAPI" 	' SAPI = simpleAPI oder CMD = ioBroker Commandline

'Zertifikate
const PEMCrtFile = "D:\Zertifikate\server.crt.pem"
const PEMKeyFile = "D:\Zertifikate\server.key.pem"

'ioBroker-Settings
const ioBrokerPath = "C:\ioBroker\"			'ioBroker-Pfad
const ioBrokerServiceName = "ioBroker.exe"	'Name des ioBroker-Dienstes
const DPKey = "admin.0.keyUpdate"			'Datenpunkt fr Key-Update
const DPCert = "admin.0.certUpdate"			'Datenpunkt fr Cert-Update

'Debug/Logging
const WriteLog = true
const LogFilename = "D:\Logs\CertUpdate-ioBroker.txt"

'################################################################
'########################## Funktionen ##########################

'Sendet Befehl zur simpleAPI um die Aktion auszufhren
Function send_command(method, action, dp, value)
	'HTTP-Request-Objekt erstellen
	Set objHTTP = CreateObject("MSXML2.XMLHTTP")

	'Befehl zusammen bauen
	command = action & "/" & dp
		
	'Wert angegeben und Methode GET, Wert in URL als Parameter anhngen
	If value <> "" and method = "GET" Then
		command = command & "?value=" & value
	End If
	
	Log "Sende Request: " & host & command
	
	On Error resume Next	'eigene Fehlerbehandlung
	objHTTP.open method, host & command, false	'synchron senden (wartet bis Request abgeschlossen und Response zurckkommt)
	
	'bei POST Daten mitsenden, sonst nicht
	If method = "POST" Then
		objHTTP.send value		'Request mit POST-Daten abschicken
	else
		objHTTP.send			'Request mit GET abschicken
	End If
	
	'Fehlerauswertung
	If Err.Number <> 0 Then
		Log vbTab & "Error: " & Err.Number
		Log vbTab & "Source: " &  Err.Source
		Log vbTab & "Description: " &  Err.Description
	Else
		Log vbTab & "HTTP-Status: " & objHTTP.status
		Log vbTab & "ResponseText: " & objHTTP.responseText

		If objHTTP.status = 200 Then
			send_command = objHTTP.ResponseText
		End If
	End If
End Function


'Update eines Datenpunkts per SAPI oder CMD
Function UpdateDP(dp, value)
	If UpdMethod = "SAPI" Then
		'per simpleAPI
		result = send_command("POST", "setValueFromBody", dp, value)
		If result <> "" Then
			success = true
		Else
			success = false
		End If
	Else
		'per Commandline
		'WSH-Shell-Objekt erstellen
		set ShellWSH = CreateObject("WScript.Shell")
		
		'Arbeitsverziechnis setzen
		ShellWSH.CurrentDirectory = ioBrokerPath
		
		'Befehl zusammen bauen
		command = "state set " & dp & " "" " & value & """"
		
		Log "Befehl ausfhen: " & command
		
		'Befehl ausfhren
		Set objExec = ShellWSH.Exec("""" & ioBrokerPath & "ioBroker.bat" & """ " & command)
		'stdout lesen
		result = objExec.stdout.ReadAll
		
		'nach Update des DP im stdout suchen (Exit-Code liefert immer 0, egal was man Updated)
		If InStr(result, "Set """ & dp & """ with value:") > 0 Then
			success = true
		Else
			success = false
		End If
	End If
	
	'frs Logging
	If success Then
		Log "Update """ & dp & """ successful!"
		UpdateDP = true
	Else
		Log "Error while updateing """ & dp & """!"
		UpdateDP = false
	End If
End Function

Function readPEMFile(file)
	'File-System-Objekt erstellen
	set fs = CreateObject("Scripting.FileSystemObject")
	
	Log "Lese PEMfile: " & file
	
	On Error resume next 	'eigene Fehlerbehandlung
	Set PEMFile = fs.opentextfile(file, 1)
	
	'Fehlerbehandlung
	If Err.Number <> 0 Then
		Log vbTab & "Error: " & Err.Number
		Log vbTab & "Source: " &  Err.Source
		Log vbTab & "Description: " &  Err.Description
		Wscript.Quit(-1) 'Skript wird beendet
	End If
	
	'Daiteiinhalt lesen
	data = PEMFile.ReadAll
	'Zeilenumbrche in druckbare Zeichen umwandeln
	data = Replace(data, vbCr, "\r")
	data = Replace(data, vbLf, "\n")
	
	readPEMFile = data
End Function

'prfen, ob ioBroker bereits luft
Function CheckioBroker()
	Set wmi = GetObject("winmgmts://./root/cimv2")
	On Error resume next
	state = wmi.Get("Win32_Service.Name='"& ioBrokerServiceName & "'").State
	
	Log "Prfe, ob ioBroker luft..."
	
	If Err.Number <> 0 Then
		Log "ioBroker-Dienst """ & ioBrokerServiceName & """ nicht gefunden!"
		Log vbTab & "Source: " &  Err.Source
		Log vbTab & "Description: " &  Err.Description
		CheckioBroker = false
	End If
	
	If LCase(state) = "running" Then
		CheckioBroker = true
	Else
		CheckioBroker = false
	End If
	
	Log "Dienst """ & ioBrokerServiceName & """ " & state
End Function

'Startet ioBroker-Dienst neu bzw. startet ihn, wenn ioBroker nicht luft
Sub Restart_ioBroker()
	'WSH-Shell-Objekt erstellen
	set ShellWSH = CreateObject("WScript.Shell")
	
	'Arbeitsverziechnis setzen
	ShellWSH.CurrentDirectory = ioBrokerPath
	
	Log "ioBroker wird (neu) gestartet..."
	set objExec = ShellWSH.Exec("cmd /K " & """" & ioBrokerPath & "serviceIoBroker.bat""" & " & exit")
	
	'warten bis Ausfhrung beendet
	do while objExec.Status = 0
	Loop
	
	'Auswertung Return-Code (nur erfolgreiche Ausfhrung der bat-Datei wird geprft)
	If objExec.ExitCode = 0 Then	
		Log "ioBroker wurde (neu) gestartet!"
	Else
		Log "Fehler beim (Neu)start von ioBroker"
	End If
End Sub

'Log-Datei anlegen, wenn nicht vorhanden, sonst ffnen und Objekt zurckgeben
Function CreateOpenLog(logfile)
	'File-System-Objekt erstellen
	set fs = CreateObject("Scripting.FileSystemObject")
	
	'Debug-File erzeugen/ffnen (append)
	If WriteLog Then
		set CreateOpenLog = fs.OpenTextFile (logfile, 8, true, 0)
	Else
		set CreateOpenLog = null
	End If
End Function

'Fgt der LogDatei einen Eintrag an
Sub Log(text)
	loglen = 120
	
	If WriteLog Then
		If Len(text) > loglen Then
			logtext = Left(text, loglen) & "..."
		else
			logtext = text
		End If
		LogFile.writeline now & ": " & logtext
	End If
End Sub

'########################## Main ##########################
set LogFile = CreateOpenLog(LogFilename)

'Beginn Log
Log "---ioBroker CertUpdate Script gestartet---"

'ioBroker starten, falls er nicht luft
If not CheckioBroker Then
	Restart_ioBroker
End If

'Zertifikate in String einlesen
PEMKeyString = readPEMFile(PEMKeyFile)
PEMCrtString = readPEMFile(PEMCrtFile)

'Update der Zertifikate durchfhren
'Key
Log "Updateing Key..."
If  not UpdateDP(DPKey, PEMKeyString) Then
	error = true
End If
'Cert
Log "Updateing Cert..."
If not UpdateDP(DPCert, PEMCrtString) Then
	error = true
End If

'wenn keine Fehler ioBroker neu starten
If not error Then
	'warte einige Zeit bis ioBroker die Zertifikate aktualisiert hat
	Wscript.sleep 5000
	Restart_ioBroker	'ioBroker neu starten
End If

'Ende Log
Log "---ioBroker CertUpdate Script beendet---"
LogFile.Close	'Logfile schlieen